#include "map.h"
#include <stdlib.h>

rect tile_shape(tile_t const *tile)
{
    if(!tile->solid) return (rect){ 0 };

    return (rect){
        .l = -fix(0.5),
        .r =  fix(0.5),
        .t = -fix(0.5),
        .b =  fix(0.5),
    };
}

tile_t *map_tile(map_t const *m, int x, int y)
{
    if((unsigned)x >= m->width || (unsigned)y >= m->height)
        return NULL;

    return &m->tiles[y * m->width + x];
}

bool map_collides(map_t const *m, rect hitbox)
{
    int y_min = ffloor(hitbox.t);
    int y_max = fceil(hitbox.b);
    int x_min = ffloor(hitbox.l);
    int x_max = fceil(hitbox.r);

    /* Collisions against walls and static objects */
    for(int y = y_min; y < y_max; y++)
    for(int x = x_min; x < x_max; x++) {
        tile_t *t = map_tile(m, x, y);
        if(!t || !t->solid) continue;

        vec2 center = { fix(x) + fix(0.5), fix(y) + fix(0.5) };
        rect tile_hitbox = rect_translate(tile_shape(t), center);

        if(rect_collide(hitbox, tile_hitbox))
            return true;
    }

    return false;
}
